#! /usr/bin/perl -w

# THIS IS A SPECIALIZED SCRIPT WITH LIMITED ERROR HANDLNG
# USE WITH CARE!

# This perl script takes account information (obtained via
# a call of &GetAccountInto) and looks at the database to
# obtain student's 
#   fullname, login, password, email
# These are returned in an array of hash references

#    This Perl script produced with the support of
#    Office of the President of the University of North Carolina
#    funded project in Computational Science, Computer Science,
#    High Performance Computing, and Information Systems:
#       Scientific Innovations in Numerical Modeling,
#       Digital Visualization, and High-speed Connectivity
#       in Western North Carolina

package SharedServer::LookupAccountInfo ;

use strict ;
use SharedServer::DBrdr ;
use DBI ;

use Exporter ;
our(@ISA, @EXPORT) ;
@ISA = qw(Exporter) ;
@EXPORT = qw(&LookupAccountInfo) ;

sub LookupAccountInfo {
    my ( $college, $system, $userList ) = @_ ;
    my $dbHandle
	= DBI->connect($dbSource, $dbUser, $dbPassword, {PrintError=>1 }) ;
    if (!defined($dbHandle)) {
	print STDERR "Unable to conect to Shared Server database\n" ;
	exit 1 ;
    }

# Obtain unique ssid from database 
    my $sqlQssidStmt =
	'SELECT ss_id FROM students WHERE college = ? AND studid = ?' ;
    my $sqlQssidPrep = $dbHandle->prepare($sqlQssidStmt) ;
    if (!defined($sqlQssidPrep)) {
	print STDERR "SQL prepare failed -- ssid lookup\n" ;
	exit 1 ;
    }

# Obtain relevent name information from database
    my $sqlQstudnameStmt
	= 'SELECT last, first, prefer FROM student_names WHERE ss_id = ?' ;
    my $sqlQstudnamePrep = $dbHandle->prepare($sqlQstudnameStmt) ;
    if (!defined($sqlQstudnamePrep)) {
	print STDERR "SQL prepare failed -- student name lookup\n" ;
	exit 1 ;
    }

# Obtain relevent account information from database
    my $sqlQstudacctStmt =
	'SELECT loginid,dfpasswd,email FROM student_accounts WHERE ss_id = ?' ;
    my $sqlQstudacctPrep = $dbHandle->prepare($sqlQstudacctStmt) ;
    if (!defined($sqlQstudacctPrep)) {
	print STDERR "SQL prepare failed -- student account lookup\n" ;
	exit 1 ;
    }

# Verify systems information from database
    my $sqlQsystemsStmt =
	'SELECT access,files FROM student_systems WHERE ss_id=? AND system=?';
    my $sqlQsystemsPrep = $dbHandle->prepare($sqlQsystemsStmt) ;
    if (!defined($sqlQsystemsPrep)) {
	print STDERR "SQL prepare failed -- student systems lookup\n" ;
	exit 1 ;
    }

    my $failed = 0 ;
    my @acctInfo = ( ) ;
    USERPROC: foreach my $user (@$userList) {
        my $row ;

	# Obtain the ssid from database
	if (!$sqlQssidPrep->execute($college, $user)) {
	    print STDERR "SQL execute query ssid failed\n" ;
	    exit 1 ;
	}
	if (!defined($row = $sqlQssidPrep->fetchrow_arrayref)) {
	    print STDERR "${user} is not in the database" ;
	    $failed = 1 ;
	    next USERPROC ;
	}
	my $ssid = $row->[0] ;

	# Obtain the "full" name for the account
	if (!$sqlQstudnamePrep->execute($ssid)) {
	    print STDERR "SQL execute query student names failed\n" ;
	    exit 1 ;
	}
	if (!defined($row = $sqlQstudnamePrep->fetchrow_arrayref)) {
	    print STDERR "${ssid} is not in the student names database" ;
	    $failed = 1 ;
	    next USERPROC ;
	}
	my $fullname ;
	if (defined($row->[2])) {
	    $fullname = $row->[2] . ' ' . $row->[0] ;
	} else {
	    $fullname = $row->[1] . ' ' . $row->[0] ;
	}

	# Obtain the login, default password, and email for the account
	if (!$sqlQstudacctPrep->execute($ssid)) {
	    print STDERR "SQL execute query student accounts failed\n" ;
	    exit 1 ;
	}
	if (!defined($row = $sqlQstudacctPrep->fetchrow_arrayref)) {
	    print STDERR "${ssid} is not in the student accounts database" ;
	    $failed = 1 ;
	    next USERPROC ;
	}
	my $login  = $row->[0] ;
	my $passwd = $row->[1] ;
	my $email  = $row->[2] ;

	# Make sure there's a reasonable entry in the systems database
	if (!$sqlQsystemsPrep->execute($ssid, $system)) {
	    print STDERR "SQL execute query student systems failed\n" ;
	    exit 1 ;
	}
	if (!defined($row = $sqlQsystemsPrep->fetchrow_arrayref)) {
	    print STDERR "${ssid} is not in the student systems database" ;
	    $failed = 1 ;
	    next USERPROC ;
	}
	my $expire = $row->[0] ;

	my %newUser = (
		       'studid'   => $user ,
		       'fullname' => $fullname ,
		       'login'    => $login ,
		       'password' => $passwd ,
		       'email'    => $email
		       ) ;
	$acctInfo[$#acctInfo+1] = \%newUser ;
    }
    return \@acctInfo ;
}
